#!/usr/bin/python

from Tkinter import *
from socket import *
import thread
import string
import Queue


class commhub:
    def __init__(self):
        self.sockobj = None
        self.host = 'localhost'
        self.port = 3002
        self.connected = 0
        self.queue = Queue.Queue()
    def connect(self, host, port):
        self.host = host
        self.port = port
        self.reconnect()
    def reconnect(self):
        if self.connected:
            print "still connected"
            return
        self.sockobj = socket(AF_INET, SOCK_STREAM)
        try:
            self.sockobj.connect((self.host,self.port))
            self.connected = 1
            self.thread = thread.start_new(self.read_thread, ())
            print "connected to", self.host, ":", self.port
        except:
            print "can't connect to", self.host, ":", self.port

    def read_thread(self):
        while self.connected:
            data = self.sockobj.recv(4096)
            if not data: break
            self.queue.put(data)

        print "connection closed"
        self.connected = 0

    def queue_poll(self):
        try:
            data = self.queue.get(0)
            return data
        except Queue.Empty:
            return None
        
    def send(self, data):
        if (self.connected):
            self.sockobj.send("%s;\n" % data)


class myscale:
    def __init__(self, parent, number):
        self.id = number
        self.ignore = 1
        self.parent = parent
        self.scale = Scale(parent.widget(), command=self.action, from_=1.0, to=0, resolution=0.001, showvalue=0)
        self.scale.pack(expand=YES, fill=Y, side=LEFT)
    def action(self, value):
        if (self.ignore):
            self.ignore -= 1
        else:
            self.parent.send("%d %f" % (self.id, self.scale.get()))

    def set(self,v):
        #ignore the action that is triggered by the set method (race)
        self.ignore = 1
        self.scale.set(v)

     

class myscalepanel:
    def __init__(self, parentwidget, name, number):
        self.parentwidget = parentwidget
        self.list = []
        self.name = name
        for i in range(number):
            self.list += [myscale(self, i)]
        hub.send( "dump %s" % name )
    def set(self, number, value):
        self.list[number].set(value)

    def widget(self):
        return self.parentwidget

    def send(self, string):
        hub.send( "%s %s" % (self.name, string))


            

class mygridbutton:
    def __init__(self, parent, number, width):
        self.id = number
        self.parent = parent
        self.value = IntVar()
        self.button = Checkbutton(parent.widget(), command=self.action, variable=self.value)
        self.button.grid(row=number/width, column=number%width, sticky = NSEW)
    def action(self):
        self.parent.send("%d %f" % (self.id, self.value.get()))

    def set(self,v):
        if (v == 1): self.value.set(1)
        else: self.value.set(0)
        

class mybuttongrid:
    def __init__(self, parentwidget, name, w=8, h=8):
        self.parentwidget = parentwidget
        self.list = []
        self.name = name
        for i in range(w*h):
            self.list += [mygridbutton(self, i, w)]
        hub.send( "dump %s" % name )
    def set(self, number, value):
        self.list[number].set(value)

    def widget(self):
        return self.parentwidget

    def send(self, string):
        hub.send( "%s %s" % (self.name, string))

class objectbutton:
    def __init__(self, parent, description):
        self.params = description.split(':')
        self.parent = parent
        self.button = Button(parent.widget(), text = self.params[0], command = self.create_object)
        self.button.pack(fill=X, expand=Y)
    def create_object(self):
        #print "creating", self.params
        p = self.params
        l = len(p)
        if l > 2:
            if p[1] == 'scalepanel':
                if l==3:
                    self.parent.scalepanel(p[0], eval(p[2]))
                else:
                    print "can't create scalepanel", p
            if p[1] == 'buttonpanel':
                if l==4:
                    self.parent.buttonpanel(p[0], eval(p[2]), eval(p[3]))
                else:
                    print "can't create buttonpanel", p
        else:
            print "can't create object", p
    
    

class controlbox:
    def __init__(self):
        self.root = Tk()
        self.windict = {}
        self.buttons = []

        #self.add_button("mixer:scalepanel:16")
        #self.add_button("vier:buttonpanel:4:4")
        #self.add_button("acht:buttonpanel:8:8")
        #self.add_button("beatbox:buttonpanel:16:4")
        #self.add_button("beatbox:buttonpanel:16:4")
        
        self.queue_poll()

    def widget(self):
        return self.root
    

    def set(self, key, number, value):
        try:
            self.windict[key].thing.set(number,value)
        except:
            print "no receiver for message:", key, number, value

    def buttonpanel(self, key, width,height):
        try:
            self.windict[key].lift()
            self.windict[key].focus()
        except:
            win = Toplevel(self.root)
            win.title(key)
            win.thing = mybuttongrid(win, key, width, height)
            win.lift()
            win.focus()
            self.windict[key] = win

    def scalepanel(self, key, width):
        try:
            self.windict[key].lift()
            self.windict[key].focus()
        except:
            win = Toplevel(self.root)
            win.title(key)
            win.thing = myscalepanel(win, key, width)
            win.lift()
            win.focus()
            self.windict[key] = win


    def queue_poll(self):
        data = hub.queue_poll()
        while data != None:
            lines = data.split(';\n')
            for line in lines:
                if line != '':
                    p = line.split()
                    l = len(p)
                    if l == 2 and p[0] == 'create':
                        self.add_button(p[1])
                    elif l == 3:
                        self.set(p[0], eval(p[1]), eval(p[2]))
                    else:
                        print "invalid command:", line,
            data = hub.queue_poll()
        self.root.after(10, self.queue_poll)
            
    def mainloop(self):
        self.root.mainloop()

    def create_object(self):
        print "wawa"

    def add_button(self, description):
        p = description.split(':')
        
        if not p[0] in self.buttons:
            # this should throw an exeption on failure!!!
            print "creating button:", p
            objectbutton(self, description)
            self.buttons += [p[0]]
            
        
        


host='localhost'
port=3002

if len(sys.argv) > 1:
    port = string.atoi(sys.argv[1])
    if len(sys.argv) > 2:
        host = sys.argv[2]

# singleton objects
hub = commhub()
cbox = controlbox()

# connect to server
hub.connect(host, port)

cbox.mainloop()
