#!/usr/bin/python

import SocketServer
import socket
import sys
import string
import thread
import time
import readline


class DataFile:
    def __init__(self):
        self.gotreadfile = 0
        self.gotwritefile = 0
        self.file = None
    def openread(self, filename):
        if not self.gotreadfile:
            self.file = open(filename, 'r')
            self.gotreadfile = 1
    def openwrite(self, filename):
        if not self.gotwritefile:
            self.file = open(filename, 'w')
            self.gotwritefile = 1
    def close(self):
        if self.gotreadfile or self.gotwritefile:
            self.file.close()
            self.gotreadfile = 0
            self.gotwritefile = 0
    
    def send(self, data):
        if self.gotwritefile:
            self.file.write(data)
            
    def receive(self):
        if self.gotreadfile:
            return  self.file.readline()



class PPanelHub:
    def __init__(self):
        self.connections = []
        self.cache = {}
        self.objects = []
        self.pd_sockobj = None
        self.pd_connected = 0
        #self.connect_to_pd('localhost', 3000)
    def add_connection(self, conn):
        self.connections += [conn]
    def delete_connection(self, conn):
        self.connections.remove(conn)
    def broadcast(self, conn, data):
        self.send_to_cache(data)
        self.send_to_pd(data)
        for i in self.connections:
            if i != conn:
                i.send(data)

    def send_to_cache(self, data):
        pdata = data.split(';')
        pdata = pdata[0].split()
        object = pdata[0]
        paramid = eval(pdata[1])
        value = eval(pdata[2])
        try:
            self.cache[object][paramid] = value
        except:
            self.cache[object] = {}
            self.cache[object][paramid] = value

    def dump_cache(self, conn, key):
        if key == 'all':
            for i in self.cache.keys():
                self.dump_cache(conn, i)
        else:
            try:
                for id in self.cache[key].keys():
                    data = "%s %s %f;\n" % (key, id, self.cache[key][id])
                    conn.send(data)
            except:
                pass

    def dump_objects(self, conn):
        for i in self.objects:
            data = "create %s;\n" % i
            conn.send(data)
        
    def add_object(self, description):
        self.objects += [description]

    def broadcast_objects(self):
        for conn in self.connections:
            self.dump_objects(conn)

    def disconnect_pd(self):
        if self.pd_connected:
            self.pd_sockobj.close()
            self.pd_connected = 0
            print "disconnected"
    

    def connect_to_pd(self, host, port):
        if self.pd_connected: return
        self.pd_sockobj =  socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self.pd_sockobj.connect((host,port))
            self.pd_connected = 1
            print "connected to pd at", host, ":", port
        except:
            print "can't connect to pd at", host, ":", port

    def send_to_pd(self, data):
        if self.pd_connected:
            self.pd_sockobj.send(data);

    #send to console
    def send(self, data):
        print data,
        
    def list_connections(self):
        for i in self.connections:
            print "%s:%d" % i.client_address

    def hangup(self):
        for i in self.connections:
            i.finish()
    
    
    


class PPanelHandler(SocketServer.BaseRequestHandler):
    def handle(self):
        print "new connection from %s:%d" % self.client_address
        hub.add_connection(self)
        hub.dump_objects(self)
        while 1:
            data = self.request.recv(4096)
            if not data: break
            comm = data.split(';')
            comm = comm[0].split()
            if comm[0] == 'dump':
                print "dump request for", comm[1], "from %s:%d" % self.client_address
                hub.dump_cache(self, comm[1])
            elif comm[0] == 'connect' and len(comm) == 3:
                hub.connect_to_pd(comm[1], eval(comm[2]))
                
            
            else:
                hub.broadcast(self, data)
        hub.delete_connection(self)
        self.request.close()
    def send(self,data):
        self.request.send(data)

hub = PPanelHub()

host = ''
pd_host = 'localhost'
pd_port = 3000


if len(sys.argv) > 1:
    port = eval(sys.argv[1])
    if len(sys.argv) > 2:
        pd_host = sys.argv[2]
        if len(sys.argv) > 3:
            pd_port = eval(sys.argv[3])

hub.connect_to_pd(pd_host, pd_port)

myaddr = (host, port)
server = SocketServer.ThreadingTCPServer(myaddr, PPanelHandler)


help={
    'dc': '(dc) dumpcache  <symbol|all>          dump the contents of the cache',
    'co': '(co) connect    <host> <port>         connect (one-way) to a pd session',
    'di': '(di) disconnect                       disconnect pd session',
    'lc': '(lc) listconn                         list inbound connections',
    'se': '(se) send       <obj> <id> <val>      send a message',
    'wr': '(wr) write      <file> <symbol|all>   write contents of cache to a file',
    'rd': '(rd) read       <file>                read contents of file into cache and broadcast',
    'cr': '(cr) createobj  <description>         create a gui object',
    'do': '(do) dumpobj                          dump gui object creation commands',
    'so': '(so) saveobj    <file>                save objects to file',
    'lo': '(lo) loadobj    <file>                load objects from file',
    
}

def console_input_thread():
    while 1:
        command = raw_input('ppserver>> ')
        c = command.split()
        l = len(c)
        if l!=0:
            if (c[0] == 'dc' or c[0] == 'dumpcache'):
                if l==2:
                    hub.dump_cache(hub, c[1])
                elif l==1:
                    hub.dump_cache(hub, 'all')
                else:
                    print "usage:", help['dc']
                    
            elif (c[0] == 'co' or c[0] == 'connect'):
                if l==3:
                    hub.connect_to_pd(c[1], eval(c[2]))
                else:
                    print "usage:", help['co']
                         
            elif (c[0] == 'di' or c[0] == 'disconnect'):
                if l==1:
                    hub.disconnect_pd()
                else:
                    print "usage:", help['di']
            elif (c[0] == 'lc') or (c[0] == 'listconn'):
                if l==1:
                    hub.list_connections()
                else:
                    print "usage:", help['lc']
            elif (c[0] == 'se') or (c[0] == 'send'):
                if l==4:
                    data = "%s %s %s;\n" % (c[1], c[2], c[3])
                    hub.broadcast(None, data)
                else:
                    print "usage:", help['se']
            elif (c[0] == 'qt') or (c[0] == 'quit'):
                if l==1:
                    hub.hangup()
            elif (c[0] == 'wr') or (c[0] == 'write'):
                if l==2 or l==3:
                    if l==3:
                        key = c[2]
                    elif l==2:
                        key = 'all'
                    datafile = DataFile()
                    datafile.openwrite(c[1])
                    hub.dump_cache(datafile, key)
                    datafile.close()
                else:
                    print "usage:", help['wr']
                    
            elif (c[0] == 'rd') or (c[0] == 'read'):
                if l==2:
                    datafile = DataFile()
                    datafile.openread(c[1])
                    try:
                        while 1:
                            data = datafile.receive()
                            hub.broadcast(None, data)
                    except:
                        pass
                    datafile.close()
                else:
                    print "usage:", help['rd']

            elif (c[0] == 'cr') or (c[0] == 'createobj'):
                print c
                if l == 2:
                    hub.add_object(c[1])
                    hub.broadcast_objects()
                else:
                    print "usage:", help['cr']
                

            elif (c[0] == 'do') or (c[0] == 'dumpobj'):
                if l == 1:
                    hub.dump_objects(hub)
                else:
                    print "usage:", help['do']
                

            elif (c[0] == 'so') or (c[0] == 'saveobj'):
                if l == 2:
                    objfile = DataFile()
                    objfile.openwrite(c[1])
                    hub.dump_objects(objfile)
                    objfile.close()
                else:
                    print "usage:", help['so']
                

            elif (c[0] == 'lo') or (c[0] == 'loadobj'):
                if l == 2:
                    objfile = DataFile()
                    objfile.openread(c[1])
                    try:
                        while 1:
                            data = objfile.receive()
                            data = data.split(';\n')
                            data = data[0].split('create ')
                            hub.add_object(data[1])
                    except:
                        pass
                    objfile.close()
                    hub.broadcast_objects()
                else:
                    print "usage:", help['so']
                

                
            elif (c[0] == 'help'):
                for key in help.keys():
                    print help[key]
            
                            
            else:
                print 'invalid command (type help for a list)'

# start input thread
thread.start_new(console_input_thread, ())

# start server
server.serve_forever()
